@@ -1,3 +1,21 @@
+# -*- coding: utf-8 -*-
+
+# uRSSus, a multiplatform GUI news agregator
+# Copyright (C) 2008 Roberto Alsina
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# version 2 as published by the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+
 import os, sys, codecs, datetime, time
 
 VERSION='0.2.10'
@@ -21,24 +39,32 @@ statusQueue=processing.Queue()
 
 feedStatusQueue=processing.Queue()
 
+# A queue where you put the feeds that you want fetched.
+# To fetch "All feeds" just pass the root feed.
+feedUpdateQueue=processing.Queue()
+
 # Configuration
 import config
 
-# Logging
-if sys.platform=='win32':
-  # easylog and Processing on windows == broken
-  def dumb(*a, **kw):
-    pass
-  critical=dumb
-  error=dumb
-  warning=dumb
-  debug=dumb
-  info=dumb 
-  setLogger=dumb 
-  DEBUG=dumb
-else:
-  from easylog import critical, error, warning, debug, info, setLogger, DEBUG
-  setLogger(name='urssus', level=DEBUG)
+import logging
+logger=logging.getLogger('urssus')
+logger.setLevel(logging.INFO)
+if sys.platform<>'win32':
+  #create console handler and set level to debug
+  ch = logging.StreamHandler()
+  ch.setLevel(logging.INFO)
+  formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
+  ch.setFormatter(formatter)
+  #add ch to logger
+  logger.addHandler(ch)
+
+critical=logger.critical
+error=logger.error
+warning=logger.warn
+info=logger.info
+debug=logger.debug
+
+logger.critical("Prueba")
 
 # Templates
 from util import tenjin
@@ -69,3 +95,21 @@ def renderTemplate(tname, **context):
 import processing
 processes=[]
 
+
+import sqlalchemy, sqlalchemy.orm
+
+session = sqlalchemy.orm.scoped_session(sqlalchemy.orm.create_session)
+
+
+def RetryOnDBError(fn):
+  def new(*args):
+    while True:
+      try:
+        return fn(*args)
+        break
+      except sqlalchemy.exc.OperationalError:
+        debug("retrying")
+        pass # Retry
+  return new
+
+
